static void 
#ifdef _CLOVER
assign_mult_Qee(const float   *kappa2,
                      cuhqfield  *yde,
                const cuhqfield   *ye,
                const cuhgvfield  *ue,
                const cuhgvfield  *uo,
                const int        *ieo,
                const cuhclvfield *fclinve,
                const cuhclvfield *fclinvo)
#else
assign_mult_Qee(const float  *kappa2,
                      cuhqfield *yde,
                const cuhqfield  *ye,
                const cuhgvfield *ue,
                const cuhgvfield *uo,
                const int       *ieo)
#endif
{
///////////////////////////////////////////////////////////////////////////////
// 
// Multiply Qee
//       
// yde = Qee ye,
//             v       v
// where Qee = Dee^dag Dee
//
//  kappa2 : hopping parameter squared (kappa^2)
//      ye : even/odd site fermion (input)
//     yde : even/odd site fermion (output)
//      ue : even/odd site gauge link
//      uo : odd/even site gauge link
//     ieo : 0 or 1 (0:even,1:odd)
// fclinve : even/odd site inverse clover term
// fclinvo : odd/even site inverse clover term
///////////////////////////////////////////////////////////////////////////////
  cuhqfield *we;
  CUDA_SAFE_CALL(cudaMalloc((void**)&we,sizeof(cuhqfield)));
#ifdef _CLOVER
  assign_mult_eoprec_wd(    kappa2, we,ye,ue,uo,ieo,fclinve,fclinvo);
  assign_mult_eoprec_dag_wd(kappa2,yde,we,ue,uo,ieo,fclinve,fclinvo);
#else
  assign_mult_eoprec_wd(    kappa2, we,ye,ue,uo,ieo);
  assign_mult_eoprec_dag_wd(kappa2,yde,we,ue,uo,ieo);
#endif
  CUDA_SAFE_CALL(cudaFree(we));
}

static void 
#ifdef _CLOVER
assign_mult_Oee(const float   *kappa2,
                      cuhqfield  *yde,
                const cuhqfield   *ye,
                const cuhgvfield  *ue,
                const cuhgvfield  *uo,
                const int         *ieo,
                const cuhclvfield *fclinve,
                const cuhclvfield *fclinvo)
#else
assign_mult_Oee(const float  *kappa2,
                      cuhqfield *yde,
                const cuhqfield  *ye,
                const cuhgvfield *ue,
                const cuhgvfield *uo,
                const int        *ieo)
#endif
{
///////////////////////////////////////////////////////////////////////////////
// 
// Multiply Oee
//       
// yde = Oee ye,
//             v   v
// where Oee = Dee Dee^dag
//
//  kappa2 : hopping parameter squared (kappa^2)
//      ye : even/odd site fermion (input)
//     yde : even/odd site fermion (output)
//      ue : even/odd site gauge link
//      uo : odd/even site gauge link
//     ieo : 0 or 1 (0:even,1:odd)
// fclinve : even/odd site inverse clover term
// fclinvo : odd/even site inverse clover term
///////////////////////////////////////////////////////////////////////////////
  cuhqfield *we;
  CUDA_SAFE_CALL(cudaMalloc((void**)&we,sizeof(cuhqfield)));
#ifdef _CLOVER
  assign_mult_eoprec_dag_wd(kappa2, we,ye,ue,uo,ieo,fclinve,fclinvo);
  assign_mult_eoprec_wd(    kappa2,yde,we,ue,uo,ieo,fclinve,fclinvo);
#else
  assign_mult_eoprec_dag_wd(kappa2, we,ye,ue,uo,ieo);
  assign_mult_eoprec_wd(    kappa2,yde,we,ue,uo,ieo);
#endif
  CUDA_SAFE_CALL(cudaFree(we));
}
