module counter_class
!
!= Definition of counter class
!
!== Version
!
!  $Id: counter_class.F90,v 1.10 2011/01/31 11:10:24 ishikawa Exp $
!
  use constants_module
  implicit none
  private
  public :: counter
  public :: new
  public :: delete
  public :: inc
  public :: get_average
  public :: get_count
  public :: get_called
  public :: is_initialized

  type counter
  !
  ! counter class
  !
    private
    real(DP)    :: ave     ! average (iter/itry)
    integer(8)  :: iter    ! counter (+inc)
    integer(8)  :: itry    ! counter (+1)
    logical :: is_initialized = .FALSE.
  end type

  interface new
  !
  ! reset counter
  !
    module procedure new_counter
  end interface

  interface delete
  !
  ! delete counter
  !
    module procedure delete_counter
  end interface

  interface inc
  !
  ! increment counter
  !
    module procedure inc_counter
  end interface

  interface get_average
  !
  ! get/compute average
  !
    module procedure get_average_counter
  end interface

  interface is_initialized
    module procedure is_initialized_counter
  end interface

  interface get_count
  !
  ! get inc count
  !
!    module procedure get_count_counter_i4
    module procedure get_count_counter_i8
  end interface

  interface get_called
  !
  ! get called count
  !
!    module procedure get_called_counter_i4
    module procedure get_called_counter_i8
  end interface

contains

subroutine new_counter(this)
  implicit none
  type(counter), intent(out) :: this
  this%ave  = 0.0_DP
  this%iter = 0
  this%itry = 0
  this%is_initialized = .TRUE.
  return
end subroutine

subroutine delete_counter(this)
  implicit none
  type(counter), intent(out) :: this
  this%ave  = 0.0_DP
  this%iter = 0
  this%itry = 0
  this%is_initialized = .FALSE.
  return
end subroutine

subroutine inc_counter(this,incl)
!
! increments counter by incl
!
! this also counts call count
!
  implicit none
  type(counter),     intent(inout) :: this  ! counter
  integer, optional, intent(in)    :: incl  ! inclement by incl

  if (present(incl)) then
    this%iter=this%iter+incl
  else
    this%iter=this%iter+1
  endif

  !
  ! count call inc 
  !
  this%itry=this%itry+1 

  return
end subroutine

function is_initialized_counter(this) result(status)
  implicit none
  type(counter), intent(in) :: this
  logical :: status
  status = this%is_initialized
  return
end function

function get_average_counter(this) result(ave)
!
! return average inclements
!
! ave <= this%iter/this%itry
!
! * this%itry : call inc count
! * this%iter : total increment count
!
  implicit none
  type(counter), intent(inout) :: this
  real(DP) :: ave

  if (this%itry /= 0) then
    this%ave=dble(this%iter)/this%itry
  else
    this%ave=0.0_DP
  endif
  ave = this%ave
  return
end function

function get_count_counter_i4(this) result(count)
!
! return current increment counter
!
  implicit none
  type(counter), intent(in) :: this
  integer :: count
  count = this%iter
  return
end function

function get_count_counter_i8(this) result(count)
!
! return current increment counter
!
  implicit none
  type(counter), intent(in) :: this
  integer(8) :: count
  count = this%iter
  return
end function

function get_called_counter_i4(this) result(count)
!
! return current call counter
!
  implicit none
  type(counter), intent(in) :: this
  integer :: count
  count = this%itry
  return
end function

function get_called_counter_i8(this) result(count)
!
! return current call counter
!
  implicit none
  type(counter), intent(in) :: this
  integer(8) :: count
  count = this%itry
  return
end function

end module
