#include <config.h>
module hmc_logfile_class
!= HMC Logfile class
!
!== Defines HMC log files
!
! Logging is done only by master node (nodeid=0) 
! file name suffix is added with hmc run number
!
!== Version
!
! $Id: hmc_logfile_class.F90,v 1.1 2011/06/14 11:10:12 ishikawa Exp $
!
  use constants_module, only : CHARLEN
  use logfile_class
  use hmc_job_manager_class
  implicit none
  private

  public :: hmc_logfile
  public :: new
  public :: delete
  public :: print
  public :: is_open
  public :: get_file_unit
  public :: set_hmc_job_manager
  public :: set_default_context
  public :: set_context

  public :: CONTEXT_RUN
  public :: CONTEXT_TRAJECTORY

  integer, parameter :: CONTEXT_RUN        = 0
  integer, parameter :: CONTEXT_TRAJECTORY = 1
  integer, parameter :: CONTEXT_NULL       = -10

  type(hmc_job_manager), save, pointer :: m_manager => NULL()
  logical, save :: m_is_initialized = .false.
  integer, save :: m_default_context = CONTEXT_RUN

  type, extends(logfile) :: hmc_logfile
  !
  ! logger object
  !
    character(len=CHARLEN), private :: fname_head
    integer,  private :: context = CONTEXT_NULL
  end type

  interface new
    module procedure new_hlog
  end interface

  interface delete
    module procedure delete_hlog
  end interface

  interface print
    module procedure print_hlog
    module procedure print_hlog_array
  end interface

  interface is_open
    module procedure is_open_hlog
  end interface

  interface get_file_unit
    module procedure get_file_unit_hlog
  end interface

  interface set_hmc_job_manager
    module procedure set_hmc_job_manager
  end interface

  interface set_default_context
    module procedure set_default_context
  end interface

  interface set_context
    module procedure set_context
  end interface

contains

subroutine set_hmc_job_manager(this,manager)
  implicit none
  type(hmc_logfile),             intent(inout) :: this
  type(hmc_job_manager), target, intent(in) :: manager
  if (.not.m_is_initialized) then
    m_manager => manager
  endif
  m_is_initialized = .true.
  return
end subroutine

subroutine set_default_context(this,context)
  use error_class
  implicit none
  type(hmc_logfile), intent(inout) :: this
  integer,            intent(in)    :: context
  select case(context)
  case (CONTEXT_RUN)
    return
  case (CONTEXT_TRAJECTORY)
    return
  case default
    call error_stop("hmc_logfile%set_default_context error.")
  end select
  m_default_context = context
  return
end subroutine

subroutine set_context(this,context)
  use error_class
  implicit none
  type(hmc_logfile), intent(inout) :: this
  integer,            intent(in)    :: context
  select case(context)
  case (CONTEXT_RUN)
    return
  case (CONTEXT_TRAJECTORY)
    return
  case default
    call error_stop("hmc_logfile%set_context error.")
  end select
  this%context = context
  return
end subroutine

function is_open_hlog(this) result(flag)
  implicit none
  type(hmc_logfile), intent(in) :: this
  logical :: flag
  flag = is_open(this%logfile)
  return
end function

function get_file_unit_hlog(this) result(iout)
  implicit none
  type(hmc_logfile), intent(in) :: this
  integer :: iout
  iout = get_file_unit(this%logfile)
  return
end function

subroutine new_hlog(this,fname)
!
! Initialize logger object and open log file
!
! -  this : logfile
! - fname : logging output file name 
!
  implicit none
  type(hmc_logfile), intent(inout) :: this
  character(len=*),  intent(in)    :: fname

  this%fname_head = fname
  this%context = m_default_context

  return
end subroutine

subroutine delete_hlog(this)
!
! delete logger object and close log file
!
! -  this : logfile
!
  implicit none
  type(hmc_logfile), intent(inout) :: this

  this%fname_head = REPEAT(' ',LEN(this%fname_head))
  call delete(this%logfile)
  this%context = m_default_context

  return
end subroutine

subroutine delay_open(this)
  use error_class
  implicit none
  type(hmc_logfile), intent(inout) :: this
  character(len=CHARLEN) :: fname,suffix
  character(len=1), parameter :: DOT_SEP = "."
  character(len=CHARLEN) :: ff
  character(len=1) :: last_char
  integer :: fname_len

  if (.not.m_is_initialized) then
    call error_stop("Do set_hmc_job_manager before using hmc_logfile.")
  endif

  if (.not.is_open(this%logfile)) then
    select case(this%context)
    case (CONTEXT_RUN)
      write(suffix,'(I6.6)') get_run_number(m_manager) 
    case (CONTEXT_TRAJECTORY)
      write(suffix,'(I6.6)') get_trajectory_number(m_manager) 
    end select

    ff = REPEAT(' ',LEN(ff))
    ff = TRIM(ADJUSTL(this%fname_head))
    fname_len = LEN(TRIM(ff))
    last_char = ff(fname_len:fname_len)
    if (.not.(DOT_SEP == last_char)) then
      fname = TRIM(ff)//DOT_SEP//TRIM(ADJUSTL(suffix))
    else
      fname = TRIM(ff)//TRIM(ADJUSTL(suffix))
    endif

    call new(this%logfile,TRIM(ADJUSTL(fname)))
  endif

  return
end subroutine

subroutine print_hlog(this,str)
!
! delete logger object and close log file
!
! -  this : logfile
! -   str : string to be printed
!
  implicit none
  type(hmc_logfile), intent(inout) :: this
  character(len=*),  intent(in) :: str

  call delay_open(this)
  call print(this%logfile,TRIM(str))

  return
end subroutine

subroutine print_hlog_array(this,str)
!
! delete logger object and close log file
!
! -  this : logfile
! -   str : string to be printed
!
  implicit none
  type(hmc_logfile), intent(inout) :: this
  character(len=*),  intent(in) :: str(:)
  integer :: ns,is

  call delay_open(this)
  call print(this%logfile,str)

!  ns = SIZE(str(:))
!  do is=1,ns
!    call print(this,str(is))
!  enddo
  return
end subroutine

end module
